//     Author: Maurizio Loreti, aka MLO or (HAM) I3NOO
//     Work:   University of Padova - Department of Physics
//             Via F. Marzolo, 8 - 35131 PADOVA - Italy
//     Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102
//     EMail:  loreti@pd.infn.it
//     WWW:    http://www.pd.infn.it/~loreti/mlo.html
//
// -------------------------------------------------------------------
// For all active board/channel values, sets the ramp-down parameter
// to 50 V/s and the voltage to 0; waits 30 seconds; then turns the
// power off, and exits.  Will be executed by the UPS daemon if the
// mains are lost.
//
// Hardware setup:
// - Power:                     ON, LOCAL
// - "Local" switch:            LOCAL ENABLE
// - "Interlock" switch:        OFF
// -------------------------------------------------------------------
// $Id: caenoff.cxx,v 1.2 2004/12/16 11:17:29 loreti Exp $
// -------------------------------------------------------------------

extern "C" {
  #include <unistd.h>
}
#include "sy2527.hh"

using std::cout;
using std::endl;
using std::string;

namespace {
  const string       name     = "Test";                 // Logical name
  const string       address  = IPADDR;                 // TCP/IP address
  const float        rampDown = 50.0;                   // Ramp down (V/s)
  const unsigned int waitTime = 30;                     // Sleep time (s)
}

int main() {
#ifndef NDEBUG
  cout << '\n';
  sy2527::version();
  cout << endl;
#endif // NDEBUG

  try {
    caenPST ps(name, address);
    unsigned int nBoards = ps.getNBoards();
    caenBD * pBD;

    for (unsigned int ib = 0;   ib < nBoards;   ++ib) {
      pBD = ps.testB(ib);
      if (pBD != 0) {
        unsigned int nChannels = pBD->getNChan();
#ifndef NDEBUG
        cout << endl;
#endif // NDEBUG
        for (unsigned int ic = 0;   ic < nChannels;   ++ic) {
          ps.setRdn(ib, ic, rampDown);
          ps.setV0(ib, ic, 0.0);
#ifndef NDEBUG
          cout << "Board " << ib << ", channel " << ic << ":\t0\n";
#endif // NDEBUG
        }
      }
    }
#ifndef NDEBUG
    cout << "Sleeping " << waitTime << " seconds:";
    cout.flush();
#endif // NDEBUG
    sleep(waitTime);
#ifndef NDEBUG
    cout << "\tdone\n";
#endif // NDEBUG
    for (unsigned int ib = 0;   ib < nBoards;   ++ib) {
      pBD = ps.testB(ib);
      if (pBD != 0) {
        unsigned int nChannels = pBD->getNChan();
        for (unsigned int ic = 0;   ic < nChannels;   ++ic) {
          ps.setOff(ib, ic);
#ifndef NDEBUG
          cout << "Board " << ib << ", channel " << ic << ":\toff\n";
#endif // NDEBUG
        }
      }
    }
  } catch (const std::exception & e) {
    std::cerr << "*** EXCEPTION:" << e.what() << endl;
  }
}
