/*------------------------------------------------------*
 | Author: Maurizio Loreti, aka MLO or (HAM) I3NOO      |
 | Work:   University of Padova - Department of Physics |
 |         Via F. Marzolo, 8 - 35131 PADOVA - Italy     |
 | Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102 |
 | EMail:  loreti@pd.infn.it                            |
 | WWW:    http://www.pd.infn.it/~loreti/mlo.html       |
 *------------------------------------------------------*

 A call to inter() establishes a signal handler for SIGINT (i.e. for a
 CTRL-C from the keyboard); the signal handler in turn just calls
 exit().  In this way all the destructors are called, all the
 allocated resources are released, and the program exits cleanly.

 *------------------------------------------------------------------*

 $Id: inter.c,v 1.2 2003/08/22 12:17:38 loreti Exp $

 *------------------------------------------------------------------*/

#include <stdio.h>              /* Standard Library */
#include <stdlib.h>
#include <unistd.h>             /* POSIX */
#include <signal.h>
#include "inter.h"              /* Local */

#if defined(__linux__)          /* OS dependent */
  #include <bits/sigaction.h>
  #include <bits/sigset.h>

  int sigemptyset();            /* Black magic?  Prototypes not seen,  */
  int sigaction();              /*   also with proper header included. */
#endif

static void sigInt(int);

static void sigInt(
  int sigNum
) {
  switch (sigNum) {
    case SIGINT:
      puts("\nInterrupted!\n");
      exit(EXIT_SUCCESS);
    default:
      printf("Unexpected signal, %d --- ignored\n", sigNum);
  }
}

void inter(void)
{
  struct sigaction action, oldAction;

  sigemptyset(&action.sa_mask);
  action.sa_handler = sigInt;
  action.sa_flags = 0;
#if defined(SA_RESTART)
  action.sa_flags |= SA_RESTART;
#endif

  if (sigaction(SIGINT, &action, &oldAction) < 0) {
    puts("Couldn't establish a SIGINT handler");
    perror("sigaction(SIGINT)");
    exit(EXIT_FAILURE);
  }
}
